@@ -2,14 +2,16 @@
 Functions focused on print outputs.
 """
 
+from dataclasses import dataclass
 from datetime import datetime
 from json import dumps as json_dumps
+from json import loads as json_loads
 from os import get_terminal_size
 from os import name as os_name
 from os.path import relpath
 from textwrap import fill
 from textwrap import indent as text_indent
-from typing import Any, Dict, List, Optional, TypeVar
+from typing import Any, Dict, List, Optional, Union
 
 from rich.console import Console, Theme
 from rich.highlighter import JSONHighlighter, RegexHighlighter
@@ -46,22 +48,32 @@ from yellowdog_client.object_store.download import DownloadBatchBuilder
 from yellowdog_client.object_store.upload import UploadBatchBuilder
 
 from yd_commands.args import ARGS_PARSER
+from yd_commands.cloudwizard_aws_types import AWSAvailabilityZone
 from yd_commands.compact_json import CompactJSONEncoder
-from yd_commands.config_keys import NAME, TASK_GROUPS, TASKS
+from yd_commands.id_utils import YDIDType
 from yd_commands.object_utilities import Item
-
-JSON_INDENT = 2
+from yd_commands.property_names import NAME, TASK_GROUPS, TASKS
+from yd_commands.settings import (
+    DEFAULT_LOG_WIDTH,
+    DEFAULT_THEME,
+    ERROR_STYLE,
+    HIGHLIGHTED_STATES,
+    JSON_INDENT,
+    MAX_LINES_COLOURED_JSON,
+    NAMESPACE_PREFIX_SEPARATOR,
+    WARNING_STYLE,
+)
 
 try:
     LOG_WIDTH = get_terminal_size().columns
 except OSError:
-    LOG_WIDTH = 120  # Default log line width
+    LOG_WIDTH = DEFAULT_LOG_WIDTH  # Default log line width
 
 
-# Set up basic Rich formatting
+# Set up Rich formatting for coloured output
 class PrintLogHighlighter(RegexHighlighter):
     """
-    Apply styles to print_log() lines.
+    Apply styles for print_log() lines.
     """
 
     base_style = "pyexamples."
@@ -70,31 +82,48 @@ class PrintLogHighlighter(RegexHighlighter):
         r" [0-9][0-9]:[0-9][0-9]:[0-9][0-9])",
         r"(?P<quoted>'[a-zA-Z0-9-._=;:\/\\\[\]{}+#@$£%\^&\*\(\)~`<>?]*')",
         r"(?P<ydid>ydid:[a-z]*:[0-9ABCDEF]*:[0-9abcdef-]*)",
-        r"(?P<url>(https?):((//)|(\\\\))+[\w\d:#@%/;$()~_?\+-=\\\.&]*)",
-    ]
+        r"(?P<url>(https?):((//)|(\\\\))+[\w\d:#@%/;$~_?\+-=\\\.&]*)",
+    ] + HIGHLIGHTED_STATES
 
 
-pyexamples_theme = Theme(
-    {
-        "pyexamples.date_time": "bold blue",
-        "pyexamples.quoted": "bold green",
-        "pyexamples.url": "bold magenta",
-        "pyexamples.ydid": "bold green",
-    }
-)
+class PrintTableHighlighter(RegexHighlighter):
+    """
+    Apply styles for table printing.
+    """
+
+    base_style = "pyexamples."
+    table_outline_chars = "┌─┬│┼┐┤└┴┘├"
+    highlights = [
+        rf"(?P<table_outline>[{table_outline_chars}]*)",
+        rf"(?P<table_content>[^{table_outline_chars}]*)",
+        r"(?P<ydid>ydid:[a-z]*:[0-9ABCDEF]*:[0-9abcdef-]*)",
+    ] + HIGHLIGHTED_STATES
+
+
+pyexamples_theme = Theme(DEFAULT_THEME)
+
 
 CONSOLE = Console(highlighter=PrintLogHighlighter(), theme=pyexamples_theme)
+CONSOLE_TABLE = Console(highlighter=PrintTableHighlighter(), theme=pyexamples_theme)
 CONSOLE_ERR = Console(stderr=True, highlighter=PrintLogHighlighter())
 CONSOLE_JSON = Console(highlighter=JSONHighlighter())
-TABLE_OUTPUT_STYLE = "bold green"
+
+PREFIX_LEN = 0
 
 
-def print_string(msg: str = "") -> str:
+def print_string(msg: str = "", no_fill: bool = False) -> str:
     """
     Message output format, with tidy line-wrapping calibrated
     for the terminal width.
     """
+    global PREFIX_LEN
     prefix = datetime.now().strftime("%Y-%m-%d %H:%M:%S") + " : "
+    if PREFIX_LEN == 0:
+        PREFIX_LEN = len(prefix)
+
+    if no_fill or msg == "" or msg.isspace() or ARGS_PARSER.no_format:
+        return prefix + msg
+
     return fill(
         msg,
         width=LOG_WIDTH,
@@ -107,9 +136,9 @@ def print_string(msg: str = "") -> str:
 
 
 def print_log(
-    log_message: str,
+    log_message: str = "",
     override_quiet: bool = False,
-    flush: bool = True,
+    no_fill: bool = False,
 ):
     """
     Placeholder for logging.
@@ -118,28 +147,42 @@ def print_log(
     if ARGS_PARSER.quiet and override_quiet is False:
         return
 
-    CONSOLE.print(print_string(log_message))
+    if ARGS_PARSER.no_format:
+        print(print_string(log_message, no_fill=no_fill), flush=True)
+        return
+
+    CONSOLE.print(print_string(log_message, no_fill=no_fill))
 
 
-ErrorObject = TypeVar(
-    "ErrorObject",
-    Exception,
-    str,
-)
-
-
-def print_error(error_obj: ErrorObject):
+def print_error(error_obj: Union[Exception, str]):
     """
     Print an error message to stderr.
     """
-    CONSOLE_ERR.print(print_string(f"Error: {error_obj}"), style="bold red")
+    if ARGS_PARSER.no_format:
+        print(print_string(f"Error: {error_obj}"), flush=True)
+        return
+
+    CONSOLE_ERR.print(print_string(f"Error: {error_obj}"), style=ERROR_STYLE)
 
 
-def print_warning(warning: str):
+def print_warning(
+    warning: str,
+    override_quiet: bool = False,
+    no_fill: bool = False,
+):
     """
     Print a warning.
     """
-    CONSOLE.print(print_string(f"Warning: {warning}"), style="red")
+    if ARGS_PARSER.quiet and override_quiet is False:
+        return
+
+    if ARGS_PARSER.no_format:
+        print(print_string(f"Warning: {warning}", no_fill=no_fill), flush=True)
+        return
+
+    CONSOLE.print(
+        print_string(f"Warning: {warning}", no_fill=no_fill), style=WARNING_STYLE
+    )
 
 
 TYPE_MAP = {
@@ -155,9 +198,20 @@ TYPE_MAP = {
     ComputeSourceTemplateSummary: "Compute Source Template",
     KeyringSummary: "Keyring",
     MachineImageFamilySummary: "Machine Image Family",
+    AWSAvailabilityZone: "AWS Availability Zones",
 }
 
 
+def print_table_core(table: str):
+    """
+    Core function for printing a table.
+    """
+    if ARGS_PARSER.no_format:
+        print(table, flush=True)
+    else:
+        CONSOLE_TABLE.print(table)
+
+
 def get_type_name(obj: Item) -> str:
     """
     Get the display name of an object's type.
@@ -186,21 +240,19 @@ def compute_requirement_table(
     ]
     table = []
     for index, cr in enumerate(cr_list):
-        table.append(
-            [
-                index + 1,
-                cr.name,
-                cr.namespace,
-                cr.tag,
-                (
-                    f"{cr.status}"
-                    if cr.nextStatus is None
-                    else (f"{cr.status} -> {cr.nextStatus}")
-                )
-                + f" ({cr.targetInstanceCount:,d}/{cr.expectedInstanceCount:,d})",
-                cr.id,
-            ]
-        )
+        table.append([
+            index + 1,
+            cr.name,
+            cr.namespace,
+            cr.tag,
+            (
+                f"{cr.status}"
+                if cr.nextStatus is None
+                else (f"{cr.status} -> {cr.nextStatus}")
+            )
+            + f" ({cr.targetInstanceCount:,d}/{cr.expectedInstanceCount:,d})",
+            cr.id,
+        ])
     return headers, table
 
 
@@ -212,16 +264,14 @@ def work_requirement_table(
     for index, wr_summary in enumerate(wr_summary_list):
         namespace = "" if wr_summary.namespace is None else wr_summary.namespace
         tag = "" if wr_summary.tag is None else wr_summary.tag
-        table.append(
-            [
-                index + 1,
-                wr_summary.name,
-                namespace,
-                tag,
-                str(wr_summary.status),
-                wr_summary.id,
-            ]
-        )
+        table.append([
+            index + 1,
+            wr_summary.name,
+            namespace,
+            tag,
+            str(wr_summary.status),
+            wr_summary.id,
+        ])
     return headers, table
 
 
@@ -231,14 +281,17 @@ def task_group_table(
     headers = ["#", "Task Group Name", "Status", "ID"]
     table = []
     for index, task_group in enumerate(task_group_list):
-        table.append(
-            [
-                index + 1,
-                task_group.name,
-                str(task_group.status),
-                task_group.id,
-            ]
-        )
+        status_msg = str(task_group.status)
+        if task_group.starved:
+            status_msg += "/STARVED"
+        if task_group.waitingOnDependency:
+            status_msg += "/WAITING"
+        table.append([
+            index + 1,
+            task_group.name,
+            status_msg,
+            task_group.id,
+        ])
     return headers, table
 
 
@@ -246,14 +299,12 @@ def task_table(task_list: List[Task]) -> (List[str], List[List]):
     headers = ["#", "Task Name", "Status", "ID"]
     table = []
     for index, task in enumerate(task_list):
-        table.append(
-            [
-                index + 1,
-                task.name,
-                str(task.status),
-                task.id,
-            ]
-        )
+        table.append([
+            index + 1,
+            task.name,
+            str(task.status),
+            task.id,
+        ])
     return headers, table
 
 
@@ -284,18 +335,14 @@ def worker_pool_table(
         node_summary: NodeSummary = worker_pool.nodeSummary
         nodes_running = node_summary.statusCounts[NodeStatus.RUNNING]
 
-        table.append(
-            [
-                index + 1,
-                worker_pool_summary.name,
-                (
-                    f"{worker_pool_summary.type.split('.')[-1:][0].replace('WorkerPool', '')}"
-                ),
-                f"{worker_pool_summary.status}",
-                f"{min_nodes}/{nodes_running}/{max_nodes}",
-                worker_pool_summary.id,
-            ]
-        )
+        table.append([
+            index + 1,
+            worker_pool_summary.name,
+            f"{worker_pool_summary.type.split('.')[-1:][0].replace('WorkerPool', '')}",
+            f"{worker_pool_summary.status}",
+            f"{min_nodes}/{nodes_running}/{max_nodes}",
+            worker_pool_summary.id,
+        ])
     return headers, table
 
 
@@ -322,16 +369,14 @@ def compute_requirement_template_table(
             )
         except:
             strategy_type = None
-        table.append(
-            [
-                index + 1,
-                crt_summary.name,
-                crt_summary.namespace,
-                type,
-                strategy_type,
-                crt_summary.id,
-            ]
-        )
+        table.append([
+            index + 1,
+            crt_summary.name,
+            crt_summary.namespace,
+            type,
+            strategy_type,
+            crt_summary.id,
+        ])
     return headers, table
 
 
@@ -356,16 +401,14 @@ def compute_source_template_table(
             provider = cst_summary.provider
         except:
             provider = None
-        table.append(
-            [
-                index + 1,
-                cst_summary.name,
-                cst_summary.namespace,
-                provider,
-                type,
-                cst_summary.id,
-            ]
-        )
+        table.append([
+            index + 1,
+            cst_summary.name,
+            cst_summary.namespace,
+            provider,
+            type,
+            cst_summary.id,
+        ])
     return headers, table
 
 
@@ -380,14 +423,12 @@ def keyring_table(
     ]
     table = []
     for index, keyring in enumerate(keyring_summaries):
-        table.append(
-            [
-                index + 1,
-                keyring.name,
-                keyring.description,
-                keyring.id,
-            ]
-        )
+        table.append([
+            index + 1,
+            keyring.name,
+            keyring.description,
+            keyring.id,
+        ])
     return headers, table
 
 
@@ -404,16 +445,14 @@ def image_family_table(
     ]
     table = []
     for index, image_family in enumerate(image_family_summaries):
-        table.append(
-            [
-                index + 1,
-                image_family.name,
-                image_family.access,
-                image_family.namespace,
-                image_family.osType,
-                image_family.id,
-            ]
-        )
+        table.append([
+            index + 1,
+            image_family.name,
+            image_family.access,
+            image_family.namespace,
+            image_family.osType,
+            image_family.id,
+        ])
     return headers, table
 
 
@@ -440,23 +479,41 @@ def instances_table(
     ]
     table = []
     for index, instance in enumerate(instances):
-        table.append(
-            [
-                index + 1,
-                instance.type.split(".")[-1],
-                instance.provider,
-                instance.instanceType,
-                instance.privateIpAddress,
-                instance.publicIpAddress,
-            ]
-        )
+        table.append([
+            index + 1,
+            instance.type.split(".")[-1],
+            instance.provider,
+            instance.instanceType,
+            instance.privateIpAddress,
+            instance.publicIpAddress,
+        ])
+    return headers, table
+
+
+def aws_availability_zone_table(
+    aws_azs: List[AWSAvailabilityZone],
+) -> (List[str], List[str]):
+    headers = [
+        "#",
+        "Availability Zone",
+        "Default Subnet ID",
+        "Default Security Group ID",
+    ]
+    table = []
+    for index, az in enumerate(aws_azs):
+        table.append([
+            index + 1,
+            az.az,
+            az.default_subnet_id,
+            az.default_sec_grp.id,
+        ])
     return headers, table
 
 
 def print_numbered_object_list(
     client: PlatformClient,
-    objects: List[Item],
-    parent: Optional[Item] = None,
+    objects: List[Union[Item, str]],
+    object_type_name: Optional[str] = None,
     override_quiet: bool = False,
     showing_all: bool = False,
 ) -> None:
@@ -469,13 +526,17 @@ def print_numbered_object_list(
 
     print_log(
         "Displaying"
-        f" {'all' if showing_all else 'matching'} {get_type_name(objects[0])}(s):",
+        f" {'all' if showing_all else 'matching'}"
+        f" {(object_type_name if object_type_name is not None else get_type_name(objects[0]))}(s):",
         override_quiet=override_quiet,
     )
     print()
 
     headers = None
-    if isinstance(objects[0], ComputeRequirement):
+    if isinstance(objects[0], str):
+        headers = ["#", "Name"]
+        table = [[index + 1, name] for index, name in enumerate(objects)]
+    elif isinstance(objects[0], ComputeRequirement):
         headers, table = compute_requirement_table(objects)
     elif isinstance(objects[0], WorkRequirementSummary):
         headers, table = work_requirement_table(objects)
@@ -497,6 +558,8 @@ def print_numbered_object_list(
         headers, table = object_path_table(objects)
     elif isinstance(objects[0], Instance):
         headers, table = instances_table(objects)
+    elif isinstance(objects[0], AWSAvailabilityZone):
+        headers, table = aws_availability_zone_table(objects)
     else:
         table = []
         for index, obj in enumerate(objects):
@@ -505,24 +568,20 @@ def print_numbered_object_list(
             except:  # Handle the Namespace Storage Configuration case
                 table.append([index + 1, ":", obj.namespace])
     if headers is None:
-        CONSOLE.print(
-            indent(tabulate(table, tablefmt="plain"), indent_width=4),
-            style=TABLE_OUTPUT_STYLE,
-        )
+        print_table_core(indent(tabulate(table, tablefmt="plain"), indent_width=4))
     else:
-        CONSOLE.print(
+        print_table_core(
             indent(
                 tabulate(table, headers=headers, tablefmt="simple_outline"),
                 indent_width=4,
-            ),
-            style=TABLE_OUTPUT_STYLE,
+            )
         )
-    print()
+    print(flush=True)
 
 
 def print_numbered_strings(objects: List[str], override_quiet: bool = False):
     """
-    Print a simple list of strings with numbering
+    Print a simple list of strings with numbering.
     """
     if ARGS_PARSER.quiet and override_quiet is False:
         return
@@ -530,14 +589,14 @@ def print_numbered_strings(objects: List[str], override_quiet: bool = False):
     table = []
     for index, obj in enumerate(objects):
         table.append([index + 1, ":", obj])
-    CONSOLE.print(
-        indent(tabulate(table, tablefmt="plain"), indent_width=4),
-        style=TABLE_OUTPUT_STYLE,
-    )
-    print()
+
+    print_table_core(indent(tabulate(table, tablefmt="plain"), indent_width=4))
+    print(flush=True)
 
 
-def sorted_objects(objects: List[Item], reverse: bool = False) -> List[Item]:
+def sorted_objects(
+    objects: List[Union[Item, str]], reverse: bool = False
+) -> List[Item]:
     """
     Sort objects by their 'name' property, or 'namespace' in the case of
     Namespace Storage Configurations, or 'instanceType' in the case of
@@ -546,9 +605,18 @@ def sorted_objects(objects: List[Item], reverse: bool = False) -> List[Item]:
     if len(objects) == 0:
         return objects
 
+    if ARGS_PARSER.reverse is not None:
+        reverse = ARGS_PARSER.reverse
+
+    if isinstance(objects[0], str):
+        return sorted(objects, reverse=reverse)
+
     if isinstance(objects[0], Instance):
         return sorted(objects, key=lambda x: x.instanceType, reverse=reverse)
 
+    if isinstance(objects[0], AWSAvailabilityZone):
+        return sorted(objects)
+
     try:
         return sorted(objects, key=lambda x: x.name, reverse=reverse)
     except:
@@ -577,10 +645,18 @@ def print_json(
     )
     if drop_first_line:
         json_string = "\n".join(json_string.splitlines()[1:])
-    if with_final_comma:
-        CONSOLE_JSON.print(json_string, end=",\n", soft_wrap=True)
+
+    # Coloured formatting of JSON console output is expensive
+    if json_string.count("\n") > MAX_LINES_COLOURED_JSON or ARGS_PARSER.no_format:
+        if with_final_comma:
+            print(json_string, end=",\n", flush=True)
+        else:
+            print(json_string, flush=True)
     else:
-        CONSOLE_JSON.print(json_string, soft_wrap=True)
+        if with_final_comma:
+            CONSOLE_JSON.print(json_string, end=",\n", soft_wrap=True)
+        else:
+            CONSOLE_JSON.print(json_string, soft_wrap=True)
 
 
 def print_yd_object(
@@ -592,7 +668,7 @@ def print_yd_object(
 ):
     """
     Print a YellowDog object as a JSON data structure,
-    using the compact JSON encoder
+    using the compact JSON encoder.
     """
     object_data: object = Json.dump(yd_object)
     if add_fields is not None:
@@ -651,7 +727,7 @@ class WorkRequirementSnapshot:
 
     def print(self):
         """
-        Print the JSON representation
+        Print the JSON representation.
         """
         print_log("Dry-run: Printing JSON Work Requirement specification:")
         print_json(self.wr_data)
@@ -660,7 +736,7 @@ class WorkRequirementSnapshot:
 
 def print_compute_template_test_result(result: ComputeRequirementTemplateTestResult):
     """
-    Print the results of a test submission of a Dynamic Compute Template
+    Print the results of a test submission of a Dynamic Compute Template.
     """
     if not isinstance(result, ComputeRequirementDynamicTemplateTestResult):
         print_log("Reports are only available for Dynamic Templates")
@@ -668,40 +744,36 @@ def print_compute_template_test_result(result: ComputeRequirementTemplateTestRes
 
     report: BestComputeSourceReport = result.report
     sources: List[BestComputeSourceReportSource] = report.sources
-    source_table = [
-        [
-            "#",
-            "Rank",
-            "Provider",
-            "Type",
-            "Region",
-            "InstanceType",
-            "Source Name",
-        ]
-    ]
+    source_table = [[
+        "#",
+        "Rank",
+        "Provider",
+        "Type",
+        "Region",
+        "InstanceType",
+        "Source Name",
+    ]]
     for index, source in enumerate(sources):
-        source_table.append(
-            [
-                index + 1,
-                source.rank,
-                source.provider,
-                source.type,
-                source.region,
-                source.instanceType,
-                source.name,
-            ]
-        )
-    print()
-    CONSOLE.print(
-        indent(tabulate(source_table, headers="firstrow", tablefmt="simple_outline")),
-        style=TABLE_OUTPUT_STYLE,
+        source_table.append([
+            index + 1,
+            source.rank,
+            source.provider,
+            source.type,
+            source.region,
+            source.instanceType,
+            source.name,
+        ])
+    print(flush=True)
+    print_table_core(
+        indent(tabulate(source_table, headers="firstrow", tablefmt="simple_outline"))
     )
-    print()
+    print(flush=True)
 
 
 def print_object_detail(object_detail: ObjectDetail):
     """
-    Pretty print an Object Detail
+    Pretty print an Object Detail.
+    Not currently used.
     """
     indent: str = 4 * " "
     print(f"{indent}Namespace:         {object_detail.namespace}")
@@ -712,7 +784,7 @@ def print_object_detail(object_detail: ObjectDetail):
 
 def print_batch_upload_files(upload_batch_builder: UploadBatchBuilder):
     """
-    Print the list of files that will be batch uploaded
+    Print the list of files that will be batch uploaded.
     """
     if ARGS_PARSER.quiet:
         return
@@ -721,58 +793,239 @@ def print_batch_upload_files(upload_batch_builder: UploadBatchBuilder):
     table = []
     # Yes, I know I shouldn't be accessing '_source_file_entries'
     for index, file_entry in enumerate(upload_batch_builder._source_file_entries):
-        table.append(
-            [
-                index + 1,
-                file_entry.source_file_path,
-                f"{upload_batch_builder.namespace}::{file_entry.default_object_name}",
-            ]
-        )
-    print()
-    print(
+        table.append([
+            index + 1,
+            file_entry.source_file_path,
+            f"{upload_batch_builder.namespace}{NAMESPACE_PREFIX_SEPARATOR}{file_entry.default_object_name}",
+        ])
+    print(flush=True)
+    print_table_core(
         indent(
-            tabulate(table, headers=headers, tablefmt="simple_outline"), indent_width=4
+            tabulate(table, headers=headers, tablefmt="simple_outline"),
+            indent_width=4,
         )
     )
-    print()
+    print(flush=True)
 
 
 def print_batch_download_files(
     download_batch_builder: DownloadBatchBuilder, flatten_downloads: bool = False
-):
+) -> int:
     """
     Print the list of files that will be batch downloaded.
+    Returns the number of files printed.
     """
     if ARGS_PARSER.quiet:
-        return
+        return 0
 
     headers = ["#", "Source Object", "Target Object"]
     directory_separator = "\\" if os_name == "nt" else "/"
     table = []
+    counter = 0
     # Yes, I know I shouldn't be accessing '_source_object_entries'
     for index, object_entry in enumerate(download_batch_builder._source_object_entries):
-        object_source = f"{object_entry.namespace}::{object_entry.object_name}"
+        object_source = f"{object_entry.namespace}{NAMESPACE_PREFIX_SEPARATOR}{object_entry.object_name}"
         object_target = (
             f"{object_entry.object_name.replace('/', directory_separator)}"
             if flatten_downloads is False
             else f"{object_entry.object_name.split('/')[-1:][0]}"
         )
-        table.append(
-            [
-                index + 1,
-                object_source,
-                relpath(
-                    f"{download_batch_builder.destination_folder}"
-                    f"{directory_separator}"
-                    f"{object_target}"
-                ),
-            ]
+        table.append([
+            index + 1,
+            object_source,
+            relpath(
+                f"{download_batch_builder.destination_folder}"
+                f"{directory_separator}"
+                f"{object_target}"
+            ),
+        ])
+        counter += 1
+
+    print(flush=True)
+
+    if ARGS_PARSER.no_format:
+        print(
+            indent(
+                tabulate(table, headers=headers, tablefmt="simple_outline"),
+                indent_width=4,
+            ),
+            flush=True,
         )
-    print()
-    CONSOLE.print(
-        indent(
-            tabulate(table, headers=headers, tablefmt="simple_outline"), indent_width=4
-        ),
-        style=TABLE_OUTPUT_STYLE,
-    )
-    print()
+    else:
+        CONSOLE_TABLE.print(
+            indent(
+                tabulate(table, headers=headers, tablefmt="simple_outline"),
+                indent_width=4,
+            ),
+        )
+    print(flush=True)
+    return counter
+
+
+@dataclass
+class StatusCount:
+    name: str
+    include_if_zero: bool = False
+
+
+STATUS_COUNTS_TASKS = [
+    StatusCount("PENDING"),
+    StatusCount("READY", True),
+    StatusCount("ALLOCATED"),
+    StatusCount("EXECUTING", True),
+    StatusCount("UPLOADING"),
+    StatusCount("DOWNLOADING"),
+    StatusCount("COMPLETED", True),
+    StatusCount("CANCELLED"),
+    StatusCount("ABORTED"),
+    StatusCount("FAILED"),
+]
+
+STATUS_COUNTS_INSTANCES = [
+    StatusCount("PENDING", True),
+    StatusCount("RUNNING", True),
+    StatusCount("STOPPING"),
+    StatusCount("STOPPED"),
+    StatusCount("TERMINATING"),
+    StatusCount("TERMINATED", True),
+    StatusCount("UNAVAILABLE"),
+    StatusCount("UNKNOWN"),
+]
+
+STATUS_COUNTS_WORKERS = [
+    StatusCount("DOING_TASK", True),
+    StatusCount("SLEEPING", True),
+    StatusCount("LATE"),
+    StatusCount("FOUND"),
+    StatusCount("LOST"),
+    StatusCount("SHUTDOWN"),
+]
+
+STATUS_COUNTS_NODES = [
+    StatusCount("RUNNING", True),
+    StatusCount("TERMINATED", True),
+    StatusCount("DEREGISTERED"),
+    StatusCount("LATE"),
+    StatusCount("LOST"),
+]
+
+STATUS_COUNTS_NODEACTIONS = [
+    # StatusCount("EMPTY", True),
+    StatusCount("WAITING", True),
+    StatusCount("EXECUTING", True),
+    StatusCount("FAILED"),
+]
+
+STATUS_COUNTS_COMPUTE_REQ = [
+    StatusCount("PENDING", True),
+    StatusCount("RUNNING", True),
+    StatusCount("STOPPING"),
+    StatusCount("STOPPED"),
+    StatusCount("TERMINATING"),
+    StatusCount("TERMINATED"),
+    StatusCount("UNAVAILABLE"),
+    StatusCount("UNKNOWN"),
+]
+
+
+def status_counts_msg(
+    status_counts: List[StatusCount],
+    counts_data: Dict,
+    empty_msg_if_zero_total: bool = False,
+) -> str:
+    """
+    Generate the count of items in specific statuses.
+    """
+    msg = ""
+    first = True
+    total_count = 0
+    for status_count in status_counts:
+        try:
+            count = counts_data[status_count.name]
+            if count > 0 or status_count.include_if_zero:
+                msg += f"{'' if first else ', '}{count:,d} {status_count.name}"
+                first = False
+                total_count += count
+        except (KeyError, TypeError):
+            continue  # Do nothing if a status is not present in the event data
+    if total_count > 0 or empty_msg_if_zero_total is False:
+        return msg
+    else:
+        return ""
+
+
+def print_event(event: str, id_type: YDIDType):
+    """
+    Print a YellowDog event.
+    """
+    data_prefix = "data:"
+
+    # Ignore events that don't have a 'data:' payload
+    if not event.startswith(data_prefix):
+        return
+
+    event_data: Dict = json_loads(event.replace(data_prefix, ""))
+
+    if ARGS_PARSER.raw_events:
+        print_json(event_data)
+        return
+
+    indent = "\n" + (" " * PREFIX_LEN) + "--> "
+    indent_2 = "\n" + (" " * (PREFIX_LEN + 4))
+
+    if id_type == YDIDType.WORK_REQ:
+        msg = f"{id_type.value} '{event_data['name']}' is {event_data['status']}"
+        for task_group in event_data["taskGroups"]:
+            status = task_group["status"]
+            if task_group["waitingOnDependency"] is True:
+                status += "/WAITING"
+            elif task_group["starved"] is True:
+                status += "/STARVED"
+            msg += (
+                f"{indent}[{status}] Task Group '{task_group['name']}':"
+                f" {task_group['taskSummary']['taskCount']:,d} Task(s){indent_2}"
+            )
+            msg += status_counts_msg(
+                STATUS_COUNTS_TASKS, task_group["taskSummary"]["statusCounts"]
+            )
+
+    elif id_type == YDIDType.WORKER_POOL:
+        msg = f"{id_type.value} '{event_data['name']}' is {event_data['status']}"
+        msg += f"{indent}Node(s):        " + status_counts_msg(
+            STATUS_COUNTS_NODES, event_data["nodeSummary"]["statusCounts"]
+        )
+        node_actions_msg = status_counts_msg(
+            STATUS_COUNTS_NODEACTIONS,
+            event_data["nodeSummary"]["actionQueueStatuses"],
+            empty_msg_if_zero_total=True,
+        )
+        if len(node_actions_msg) > 0:
+            msg += f"{indent}Node Action(s): " + node_actions_msg
+        workers_msg = status_counts_msg(
+            STATUS_COUNTS_WORKERS,
+            event_data["workerSummary"]["statusCounts"],
+            empty_msg_if_zero_total=True,
+        )
+        if len(workers_msg) > 0:
+            msg += f"{indent}Worker(s):      " + workers_msg
+
+    elif id_type == YDIDType.COMPUTE_REQ:
+        msg = f"{id_type.value} '{event_data['name']}' is {event_data['status']}"
+        msg += (
+            f"{indent}Instance(s): "
+            f"{event_data['targetInstanceCount']:,d} TARGET,"
+            f" {event_data['expectedInstanceCount']:,d} EXPECTED"
+        )
+        for source in event_data["provisionStrategy"]["sources"]:
+            source_msg = status_counts_msg(
+                STATUS_COUNTS_INSTANCES,
+                source["instanceSummary"]["statusCounts"],
+                empty_msg_if_zero_total=True,
+            )
+            if len(source_msg) > 0:
+                msg += f"{indent}Source: '{source['name']}': " + source_msg
+
+    else:
+        return
+
+    print_log(msg, no_fill=True)
