@@ -70,10 +70,16 @@ GENERATED_ID = generate_id("wp" + "_" + CONFIG_COMMON.name_tag)
 
 @main_wrapper
 def main():
+
+    # Direct file > file supplied using '-p' > file supplied in config file
     wp_json_file = (
-        CONFIG_WP.worker_pool_data_file
-        if ARGS_PARSER.worker_pool_file is None
-        else ARGS_PARSER.worker_pool_file
+        (
+            CONFIG_WP.worker_pool_data_file
+            if ARGS_PARSER.worker_pool_file is None
+            else ARGS_PARSER.worker_pool_file
+        )
+        if ARGS_PARSER.worker_pool_file_positional is None
+        else ARGS_PARSER.worker_pool_file_positional
     )
 
     # Where do we find the data files?
@@ -103,6 +109,10 @@ def create_worker_pool_from_json(wp_json_file: str) -> None:
             wp_json_file, prefix=WP_VARIABLES_PREFIX, postfix=WP_VARIABLES_POSTFIX
         )
     else:
+        if ARGS_PARSER.jsonnet_dry_run:
+            raise Exception(
+                f"Option '--jsonnet-dry-run' can only be used with files ending in '.jsonnet'"
+            )
         wp_data = load_json_file_with_variable_substitutions(
             wp_json_file, prefix=WP_VARIABLES_PREFIX, postfix=WP_VARIABLES_POSTFIX
         )
